@@ -21,6 +21,7 @@
 */
 
 #include <config.h>
+#include <glib/gi18n.h>
 #include <string.h>
 #include <bonobo.h>
 
@@ -33,7 +34,7 @@
 #include "htmlundo.h"
 
 #include "engine.h"
-#include "spell.h"
+#include "spellchecker.h"
 
 static BonoboObjectClass *engine_parent_class;
 
